/* Filename: MAILING.C
   * Mailing List Application *
     ------------------------

   This program enables the user to enter, edit, maintain,
   and print a mailing list of names and addresses.

   All commands and concepts included in this program are
   explained throughout the text of C By Example.        */

/* Header files used by the program */
/* These header files contain function prototypes
   needed by the called functions. */
#include <stdio.h>
#include <ctype.h>

/* Global structure of a name and address */
struct mail_struct
   { char name[25];            /* Name stored here should
                                  be last, first order */
     char address[25];
     char city[12];
     char state[3];          /* Save room for null zero */
     char zipcode[6];
     char code[7];    /* For additional expansion. You
                         might want to use this member for
                         customer codes, vendor codes, or
                         holiday card codes */
   };
/* Global file pointer */
FILE *fp;

/* Defined constants */
/* MAX is the total number of names allowed
    in memory for reading mailing list */
#define MAX 250
#define BELL '\x07'
#define FILENAME "ADDRESS.DAT"

/* Prototype all of this program's functions */
void main(void);
void disp_menu(void);
void clear_sc(void);
void enter_na(void);
void change_na(void);
void print_na(void);
void err_msg(char err_msg[]);
void add_to_file(struct mail_struct item);
void pause_sc(void);
void pr_data(struct mail_struct item);
void get_new_item(struct mail_struct *item);
void getzip(char zipcode[]);

/*********************************************************/
void main(void)
{
   struct mail_struct list[MAX];
   char ans;
   do
     { disp_menu();      /* Display menu for the user */
       scanf(" %d", &ans);
       switch (ans)
       {  case (1) : { enter_na();
                 break; }
       case (2) : { change_na();
                 break; }
       case (3) : { print_na();
                 break; }
       case (4) : { break; }
       default  : { err_msg("*** You need to enter");
                    printf(" 1 through 4 ***");
                 break; }
       }
    } while (ans!=4);
   return 0;
}

/*********************************************************/
void disp_menu()  /* Display main menu of the program */
{
   clear_sc();             /* Clear either the screen
                              or the output text window */
   printf("\t\t*** Mailing List Manager ***\n");
   printf("\t\t    --------------------\n\n\n\n");
   printf("Do you want to:\n\n\n");
   printf("\t1. Add names and addresses to the list\n\n\n");
   printf("\t2. Change names and addresses in the list\n\
           \n\n");
   printf("\t3. Print names and addresses in the list\n\
           \n\n");
   printf("\t4. Exit this program\n\n\n");
   printf("What is your choice? ");
   return 0;
}

/*********************************************************/
void clear_sc()            /* Clear the screen by sending
                                   25 blank lines to it */
   /** Your compiler probably has a compiler-specific
       function that clears the screen in a more elegant
       style. This program is generic for all types of
       computers (PCs or mainframes), so the screen is
       cleared here by printing several newlines **/
{
   int ctr;           /* Counter for the 25 blank lines */
   for(ctr=0;ctr<25;ctr++)
      { printf("\n"); }
   return 0;
}

/*********************************************************/
void enter_na(void)
{
   struct mail_struct item;
   char ans;
   do
   { fflush(stdin);               /* Clear input buffer
                                  before getting strings */
     printf("\n\n\n\n\nWhat is the name? ");
     gets(item.name);
     printf("What is the address? ");
     gets(item.address);
     printf("What is the city? ");
     gets(item.city);
     printf("What is the state? (2-letter abbreviation \
             only) ");
     gets(item.state);

     getzip(item.zipcode);      /* Ensure that ZIP code
                                       is all digits */

     strcpy(item.code, " ");  /* Null out code member */
     add_to_file(item);        /* Write new information
                                         to disk file */
     printf("\n\nDo you want to enter another name");
     printf(" and address? (Y/N) ");
     ans=getchar();
     getchar();             /* Discard carriage return */
   } while (toupper(ans)=='Y');

   return 0;
}

/*********************************************************/
void change_na(void)
{
   /* This search function can be improved by using the  */
   /* code member to assign a unique code to each person */
   /* in the list. Names are difficult to search for     */
   /* because there are so many variations (such as      */
   /* Mc and Mac, and St. and Saint.                     */

   struct mail_struct item;
   int ans, s;               /* Holds size of structure */
   int change_yes = 0;           /* Will be True if user
                                  finds a name to change */
   char test_name[25];

   printf("\nWhat is the name of the person you want");
   printf(" to change? ");
   fflush(stdin);   /* Clear out all in input buffer */
   gets(test_name);
   fflush(stdin);

   s = sizeof(struct mail_struct);   /* To ensure that
                                fread() reads properly */
   if((fp=fopen(FILENAME, "r+"))==NULL)
      { err_msg("*** Read error--ensure file
                exists ***");
     return 0; }
   do
     { if(fread(&item, sizeof(struct mail_struct), 1,
                fp)!=s)
      { if(feof(fp))          /* If EOF, quit reading */
          { break; }
      }
       if (strcmp(item.name, test_name)==0)
       {  pr_data(item);    /* Print name and address */
          printf("\nIs this the name and address to");
          printf(" change? (Y/N) ");
          ans=getchar();
          getchar();       /* Discard carriage return */
          if (toupper(ans)=='N')
            { continue; }         /* Get another name */
          get_new_item(&item);     /* Let user type new
                                          information */
          fseek(fp, (long)-s, SEEK_CUR);   /* Back up
                                        one structure */
          fwrite(&item, s, 1, fp);        /* Rewrite
                                          information */
          change_yes = 1;             /* Changed flag */
          break;                          /* Finished */
       }
     } while (!feof(fp));
   fclose(fp);
   if (!change_yes)
     { err_msg("*** End of file--couldn't find\
                name ***"); }
   return 0;
}

/*********************************************************/
void print_na(void)
{
   struct mail_struct item;
   int s, linectr=0;

   s = sizeof(struct mail_struct);   /* To ensure that
                                fread() reads properly */
   if((fp=fopen(FILENAME, "r"))==NULL)
      { err_msg("*** Read error--ensure \
                 file exists ***");
     return 0; }
   do
     { if(fread(&item, sizeof(struct mail_struct), 1,
                fp)!=s)
      { if(feof(fp))          /* If EOF, quit reading */
          { break; }
      }
       if (linectr>20)              /* Screen is full */
      { pause_sc();
        linectr=0; }
       pr_data(item);    /* Print name and address */
       linectr+=4;
     } while (!feof(fp));
   fclose(fp);
   printf("\n- End of list -");
   pause_sc();           /* Give user a chance to see
                            names remaining onscreen */
   return 0;
}

/*********************************************************/
void err_msg(char err_msg[])
{
   printf("\n\n%s\n", err_msg);
   printf("%c", BELL);
   return 0;
}

/*********************************************************/
void add_to_file(struct mail_struct item)
{
   if((fp = fopen(FILENAME, "a"))==NULL)  /* Open file in
                                           append mode */
      { err_msg("*** Disk error--please check \
                 disk drive ***");
     return 0; }
   fwrite(&item, sizeof(item), 1, fp);   /* Add structure
                                           to the file */
   fclose(fp);
   return 0;
}

/*********************************************************/
void pause_sc()
{
   char ans;
   fflush(stdin);          /* Remove any extra characters
                                 from the input buffer */
   printf("\nPress the Enter key to continue...");
   ans=getchar();
   fflush(stdin);          /* Remove any extra characters
                                 from the input buffer */
   return 0;
}

/*********************************************************/
void pr_data(struct mail_struct item)
{
    /* Prints the name and address sent to it */
    printf("\nName   : %-25s\n", item.name);
    printf("Address: %-25s\n", item.address);
    printf("City   : %-12s\tState: %-2s  Zipcode: %-5s\n",
           item.city, item.state, item.zipcode);
    return 0;
}

/*********************************************************/
void get_new_item(struct mail_struct *item)
{
   struct mail_struct temp_item;    /* Holds temporary
                                       and changed input */
   printf("\nEnter new name and address information");
   printf(" below\n(Press the ");
   printf("Enter key without typing data to retain old");
   printf(" information)\n\n");
   printf("What is the new name? ");
   if (strlen(gets(temp_item.name))>0)   /* Save new data
                                          only if user */
     { strcpy((*item).name, temp_item.name); } /* presses
                                             Enter key */
   printf("What is the address? ");
   if (strlen(gets(temp_item.address))>0)
     { strcpy((*item).address, temp_item.address); }
   printf("What is the city? ");
   if (strlen(gets(temp_item.city))>0)
     { strcpy((*item).city, temp_item.city); }
   printf("What is the state? (2-letter abbreviation \
           only) ");
   if (strlen(gets(temp_item.state))>0)
     { strcpy((*item).state, temp_item.state); }
   printf("What is the ZIP code? ");
   if (strlen(gets(temp_item.zipcode))>0)
     { strcpy((*item).zipcode, temp_item.zipcode); }
   (*item).code[0] = 0;   /* Null out code member
                             (unused here) */

   return 0;  /* Return structure with changed member(s) */
}

void getzip(char zipcode[])       /* Ensure that ZIP code
                                         is all digits */
{
   int ctr, bad_zip;

   do
     { bad_zip = 0;
       printf("What is the ZIP code? ");
       gets(zipcode);
       for(ctr=0;ctr<5;ctr++)
      { if (isdigit(zipcode[ctr]))
          { continue; }
        else
           { err_msg("*** The ZIP code must be digits \
                      only ***");
          bad_zip = 1;
          break; }
      }
     } while (bad_zip);
   return 0;
}

